@@ -14,9 +14,9 @@
 
 #pragma once
 
-#include "dense_enum_map.h"
-#include "log.h"
-#include "spare_enum_map.h"
+#include "common/dense_enum_map.h"
+#include "common/spare_enum_map.h"
+#include "common/spdlog.h"
 
 #include <cstdint>
 #include <string_view>
@@ -42,8 +42,7 @@ static inline constexpr auto WasmPhaseStr = []() constexpr {
 #undef UseWasmPhase
   };
   return DenseEnumMap(Array);
-}
-();
+}();
 
 /// Error category C++ enumeration class.
 enum class ErrCategory : uint8_t {
@@ -79,14 +78,15 @@ public:
   constexpr WasmPhase getErrCodePhase() const noexcept {
     return getCategory() != ErrCategory::WASM
                ? WasmPhase::UserDefined
-               : static_cast<WasmPhase>((getCode() & 0xF0U) >> 5);
+               : static_cast<WasmPhase>((getCode() >> 8) & 0x0FU);
   }
 
   constexpr ErrCode() noexcept : Inner(0) {}
   constexpr ErrCode(const ErrCode &E) noexcept : Inner(E.Inner.Num) {}
   constexpr ErrCode(const ErrCode::Value E) noexcept : Inner(E) {}
   constexpr ErrCode(const uint32_t N) noexcept
-      : Inner(N & 0x00FFFFFFU) {}
+      : Inner((static_cast<uint32_t>(ErrCategory::UserLevelError) << 24) +
+              (N & 0x00FFFFFFU)) {}
   constexpr ErrCode(const ErrCategory C, const uint32_t N) noexcept
       : Inner((static_cast<uint32_t>(C) << 24) + (N & 0x00FFFFFFU)) {}
 
@@ -119,8 +119,8 @@ public:
 
 private:
   union InnerT {
-    constexpr InnerT(uint32_t Num): Num(Num) {}
-    constexpr InnerT(ErrCode::Value Code): Code(Code) {}
+    constexpr InnerT(uint32_t Num) : Num(Num) {}
+    constexpr InnerT(ErrCode::Value Code) : Code(Code) {}
     uint32_t Num;
     ErrCode::Value Code;
   } Inner;
@@ -136,8 +136,7 @@ static inline constexpr const auto ErrCodeStr = []() constexpr {
 #undef UseErrCode
   };
   return SpareEnumMap(Array);
-}
-();
+}();
 
 } // namespace WasmEdge
 
