@@ -53,21 +53,30 @@ struct FdHolder {
   FdHolder(const FdHolder &) = delete;
   FdHolder &operator=(const FdHolder &) = delete;
   FdHolder(FdHolder &&RHS) noexcept
-      : Fd(std::exchange(RHS.Fd, -1)), Cleanup(RHS.Cleanup) {}
+      : Fd(std::exchange(RHS.Fd, -1)), Cleanup(RHS.Cleanup),
+        Append(RHS.Append) {
+    RHS.Cleanup = true;
+    RHS.Append = false;
+  }
   FdHolder &operator=(FdHolder &&RHS) noexcept {
     using std::swap;
     swap(Fd, RHS.Fd);
+    Cleanup = RHS.Cleanup;
+    Append = RHS.Append;
+    RHS.Cleanup = true;
+    RHS.Append = false;
     return *this;
   }
 
-  constexpr FdHolder() noexcept = default;
+  constexpr FdHolder() noexcept : Fd(-1), Cleanup(true), Append(false) {}
   ~FdHolder() noexcept {
     if (Cleanup) {
       reset();
     }
   }
-  explicit constexpr FdHolder(int Fd, bool Cleanup = true) noexcept
-      : Fd(Fd), Cleanup(Cleanup) {}
+  explicit constexpr FdHolder(int Fd, bool Cleanup = true,
+                              bool Append = false) noexcept
+      : Fd(Fd), Cleanup(Cleanup), Append(Append) {}
   constexpr bool ok() const noexcept { return Fd >= 0; }
   void reset() noexcept;
   int release() noexcept { return std::exchange(Fd, -1); }
@@ -77,7 +86,8 @@ struct FdHolder {
   }
   int getFd() noexcept { return Fd; }
   int Fd = -1;
-  bool Cleanup = true;
+  bool Cleanup : 1;
+  mutable bool Append : 1;
 };
 
 struct DirHolder {
