@@ -31,18 +31,20 @@ public:
   ~Validator() noexcept = default;
 
   /// Validate AST::Component.
-  Expect<void> validate(const AST::Component &Comp);
+  Expect<void> validate(const AST::Component::Component &Comp);
   /// Validate AST::Module.
   Expect<void> validate(const AST::Module &Mod);
 
 private:
   /// Validate AST::Types
+  Expect<void> validate(const AST::SubType &Type);
   Expect<void> validate(const AST::Limit &Lim);
   Expect<void> validate(const AST::TableType &Tab);
   Expect<void> validate(const AST::MemoryType &Mem);
-  /// GlobalType is always valid.
+  Expect<void> validate(const AST::GlobalType &Glob);
 
   /// Validate AST::Segments
+  Expect<void> validate(const AST::TableSegment &TabSeg);
   Expect<void> validate(const AST::GlobalSegment &GlobSeg);
   Expect<void> validate(const AST::ElementSegment &ElemSeg);
   Expect<void> validate(const AST::CodeSegment &CodeSeg,
@@ -54,6 +56,7 @@ private:
   Expect<void> validate(const AST::ExportDesc &ExpDesc);
 
   /// Validate AST::Sections
+  Expect<void> validate(const AST::TypeSection &TypeSec);
   Expect<void> validate(const AST::ImportSection &ImportSec);
   Expect<void> validate(const AST::FunctionSection &FuncSec);
   Expect<void> validate(const AST::TableSection &TabSec);
@@ -64,6 +67,7 @@ private:
   Expect<void> validate(const AST::DataSection &DataSec);
   Expect<void> validate(const AST::StartSection &StartSec);
   Expect<void> validate(const AST::ExportSection &ExportSec);
+  Expect<void> validate(const AST::TagSection &TagSec);
 
   /// Validate const expression
   Expect<void> validateConstExpr(AST::InstrView Instrs,
