@@ -232,8 +232,8 @@ private:
                     Span<const ValVariant> Params = {},
                     Span<const ValType> ParamTypes = {});
   Expect<std::vector<std::pair<ValVariant, ValType>>>
-  unsafeRunWasmFile(const AST::Component &Component, std::string_view Func,
-                    Span<const ValVariant> Params = {},
+  unsafeRunWasmFile(const AST::Component::Component &Component,
+                    std::string_view Func, Span<const ValVariant> Params = {},
                     Span<const ValType> ParamTypes = {});
 
   Expect<void> unsafeLoadWasm(const std::filesystem::path &Path);
