@@ -87,9 +87,6 @@ constexpr int openFlags(__wasi_oflags_t OpenFlags, __wasi_fdflags_t FdFlags,
   }
 
   // Convert file descriptor flags.
-  if ((FdFlags & __WASI_FDFLAGS_APPEND) != 0) {
-    Flags |= O_APPEND;
-  }
   if ((FdFlags & __WASI_FDFLAGS_DSYNC) != 0) {
 #ifdef O_DSYNC
     Flags |= O_DSYNC;
@@ -171,7 +168,8 @@ WasiExpect<INode> INode::open(std::string Path, __wasi_oflags_t OpenFlags,
   if (auto NewFd = ::open(Path.c_str(), Flags, 0644); unlikely(NewFd < 0)) {
     return WasiUnexpect(fromErrNo(errno));
   } else {
-    INode New(NewFd);
+    INode New(NewFd, true, FdFlags & __WASI_FDFLAGS_APPEND);
+
 #ifndef O_CLOEXEC
     if (auto Res = ::fcntl(New.Fd, F_SETFD, FD_CLOEXEC); unlikely(Res != 0)) {
       return WasiUnexpect(fromErrNo(errno));
@@ -195,7 +193,9 @@ WasiExpect<void> INode::fdAdvise(__wasi_filesize_t Offset,
 WasiExpect<void> INode::fdAllocate(__wasi_filesize_t Offset,
                                    __wasi_filesize_t Len) const noexcept {
   if (auto Res = ::posix_fallocate(Fd, Offset, Len); unlikely(Res != 0)) {
-    return WasiUnexpect(fromErrNo(errno));
+    // https://man7.org/linux/man-pages/man3/posix_fallocate.3.html
+    // posix_fallocate will not set errno, use return the value directly.
+    return WasiUnexpect(fromErrNo(Res));
   }
 
   return {};
@@ -220,7 +220,7 @@ WasiExpect<void> INode::fdFdstatGet(__wasi_fdstat_t &FdStat) const noexcept {
     FdStat.fs_filetype = unsafeFiletype();
 
     FdStat.fs_flags = static_cast<__wasi_fdflags_t>(0);
-    if (FdFlags & O_APPEND) {
+    if (Append) {
       FdStat.fs_flags |= __WASI_FDFLAGS_APPEND;
     }
     if (FdFlags & O_DSYNC) {
@@ -243,9 +243,6 @@ INode::fdFdstatSetFlags(__wasi_fdflags_t FdFlags) const noexcept {
   if (FdFlags & __WASI_FDFLAGS_NONBLOCK) {
     SysFlag |= O_NONBLOCK;
   }
-  if (FdFlags & __WASI_FDFLAGS_APPEND) {
-    SysFlag |= O_APPEND;
-  }
   if (FdFlags & __WASI_FDFLAGS_DSYNC) {
     SysFlag |= O_DSYNC;
   }
@@ -260,6 +257,7 @@ INode::fdFdstatSetFlags(__wasi_fdflags_t FdFlags) const noexcept {
     return WasiUnexpect(fromErrNo(errno));
   }
 
+  Append = FdFlags & __WASI_FDFLAGS_APPEND;
   return {};
 }
 
@@ -578,6 +576,10 @@ WasiExpect<void> INode::fdWrite(Span<Span<const uint8_t>> IOVs,
     ++SysIOVsSize;
   }
 
+  if (Append) {
+    ::lseek(Fd, 0, SEEK_END);
+  }
+
   if (auto Res = ::writev(Fd, SysIOVs, SysIOVsSize); unlikely(Res < 0)) {
     return WasiUnexpect(fromErrNo(errno));
   } else {
@@ -610,7 +612,7 @@ INode::pathFilestatGet(std::string Path,
 
   Filestat.dev = SysFStat.st_dev;
   Filestat.ino = SysFStat.st_ino;
-  Filestat.filetype = fromFileType(SysFStat.st_mode);
+  Filestat.filetype = fromFileType(static_cast<mode_t>(SysFStat.st_mode));
   Filestat.nlink = SysFStat.st_nlink;
   Filestat.size = SysFStat.st_size;
   Filestat.atim = fromTimespec(SysFStat.st_atim);
@@ -733,7 +735,8 @@ WasiExpect<INode> INode::pathOpen(std::string Path, __wasi_oflags_t OpenFlags,
       unlikely(NewFd < 0)) {
     return WasiUnexpect(fromErrNo(errno));
   } else {
-    INode New(NewFd);
+    INode New(NewFd, true, FdFlags & __WASI_FDFLAGS_APPEND);
+
 #ifndef O_CLOEXEC
     if (auto Res = ::fcntl(New.Fd, F_SETFD, FD_CLOEXEC); unlikely(Res != 0)) {
       return WasiUnexpect(fromErrNo(errno));
@@ -1371,7 +1374,7 @@ INode::sockGetPeerAddr(__wasi_address_family_t *AddressFamilyPtr,
 }
 
 __wasi_filetype_t INode::unsafeFiletype() const noexcept {
-  return fromFileType(Stat->st_mode);
+  return fromFileType(static_cast<mode_t>(Stat->st_mode));
 }
 
 WasiExpect<__wasi_filetype_t> INode::filetype() const noexcept {
@@ -1857,13 +1860,16 @@ void Poller::wait() noexcept {
     const auto Iter = FdDatas.find(EPollEvent.data.fd);
     assuming(Iter != FdDatas.end());
 
-    if (EPollEvent.events & EPOLLIN) {
+    const bool NoInOut = !(EPollEvent.events & (EPOLLIN | EPOLLOUT));
+    if ((EPollEvent.events & EPOLLIN) ||
+        (NoInOut && EPollEvent.events & EPOLLHUP && Iter->second.ReadEvent)) {
       assuming(Iter->second.ReadEvent);
       assuming(Iter->second.ReadEvent->type == __WASI_EVENTTYPE_CLOCK ||
                Iter->second.ReadEvent->type == __WASI_EVENTTYPE_FD_READ);
       ProcessEvent(EPollEvent, *Iter->second.ReadEvent);
     }
-    if (EPollEvent.events & EPOLLOUT) {
+    if (EPollEvent.events & EPOLLOUT ||
+        (NoInOut && EPollEvent.events & EPOLLHUP && Iter->second.WriteEvent)) {
       assuming(Iter->second.WriteEvent);
       assuming(Iter->second.WriteEvent->type == __WASI_EVENTTYPE_FD_WRITE);
       ProcessEvent(EPollEvent, *Iter->second.WriteEvent);
