@@ -9,9 +9,10 @@
 #include <cstddef>
 #include <fstream>
 #include <limits>
-#include <string_view>
+#include <memory>
 #include <system_error>
 #include <utility>
+#include <variant>
 
 namespace WasmEdge {
 namespace Loader {
@@ -60,7 +61,8 @@ Loader::loadFile(const std::filesystem::path &FilePath) {
   return Buf;
 }
 
-Expect<std::variant<AST::Component, AST::Module>>
+Expect<std::variant<std::unique_ptr<AST::Component::Component>,
+                    std::unique_ptr<AST::Module>>>
 Loader::parseWasmUnit(const std::filesystem::path &FilePath) {
   std::lock_guard Lock(Mutex);
   // Set path and check the header.
@@ -80,11 +82,12 @@ Loader::parseWasmUnit(const std::filesystem::path &FilePath) {
     // AOT compiled shared-library-WASM cases. Use ldmgr to load the module.
     WASMType = InputType::SharedLibrary;
     FMgr.reset();
-    if (auto Res = LMgr.setPath(FilePath); !Res) {
+    std::shared_ptr<SharedLibrary> Library = std::make_shared<SharedLibrary>();
+    if (auto Res = Library->load(FilePath); !Res) {
       spdlog::error(ErrInfo::InfoFile(FilePath));
       return Unexpect(Res);
     }
-    if (auto Res = LMgr.getVersion()) {
+    if (auto Res = Library->getVersion()) {
       if (*Res != AOT::kBinaryVersion) {
         spdlog::error(ErrInfo::InfoMismatch(AOT::kBinaryVersion, *Res));
         spdlog::error(ErrInfo::InfoFile(FilePath));
@@ -96,7 +99,7 @@ Loader::parseWasmUnit(const std::filesystem::path &FilePath) {
     }
 
     std::unique_ptr<AST::Module> Mod;
-    if (auto Code = LMgr.getWasm()) {
+    if (auto Code = Library->getWasm()) {
       // Set the binary and load module.
       // Not to use parseModule() here to keep the `WASMType` value.
       if (auto Res = FMgr.setCode(*Code); !Res) {
@@ -104,8 +107,8 @@ Loader::parseWasmUnit(const std::filesystem::path &FilePath) {
         return Unexpect(Res);
       }
       if (auto Res = loadUnit()) {
-        if (std::holds_alternative<AST::Module>(*Res)) {
-          Mod = std::make_unique<AST::Module>(std::get<AST::Module>(*Res));
+        if (std::holds_alternative<std::unique_ptr<AST::Module>>(*Res)) {
+          Mod = std::move(std::get<std::unique_ptr<AST::Module>>(*Res));
         }
       } else {
         spdlog::error(ErrInfo::InfoFile(FilePath));
@@ -118,12 +121,12 @@ Loader::parseWasmUnit(const std::filesystem::path &FilePath) {
     if (!Conf.getRuntimeConfigure().isForceInterpreter()) {
       // If the configure is set to force interpreter mode, not to load the AOT
       // related data.
-      if (auto Res = loadCompiled(*Mod.get()); unlikely(!Res)) {
+      if (auto Res = loadExecutable(*Mod, Library); unlikely(!Res)) {
         spdlog::error(ErrInfo::InfoFile(FilePath));
         return Unexpect(Res);
       }
     }
-    return *Mod.get();
+    return Mod;
   }
   default:
     // Universal WASM, WASM, or other cases. Load and parse the module directly.
@@ -138,11 +141,11 @@ Loader::parseWasmUnit(const std::filesystem::path &FilePath) {
       return Unit;
     case 1: // module
     default: {
-      auto Mod = std::get<AST::Module>(*Unit);
+      auto Mod = std::move(std::get<std::unique_ptr<AST::Module>>(*Unit));
       if (!Conf.getRuntimeConfigure().isForceInterpreter()) {
         // If the configure is set to force interpreter mode, not to set the
         // symbol.
-        if (auto &Symbol = Mod.getSymbol()) {
+        if (auto &Symbol = Mod->getSymbol()) {
           *Symbol = IntrinsicsTable;
         }
       }
@@ -152,7 +155,8 @@ Loader::parseWasmUnit(const std::filesystem::path &FilePath) {
   }
 }
 
-Expect<std::variant<AST::Component, AST::Module>>
+Expect<std::variant<std::unique_ptr<AST::Component::Component>,
+                    std::unique_ptr<AST::Module>>>
 Loader::parseWasmUnit(Span<const uint8_t> Code) {
   std::lock_guard Lock(Mutex);
   if (auto Res = FMgr.setCode(Code); !Res) {
@@ -165,6 +169,7 @@ Loader::parseWasmUnit(Span<const uint8_t> Code) {
   case FileMgr::FileHeader::DLL:
   case FileMgr::FileHeader::MachO_32:
   case FileMgr::FileHeader::MachO_64:
+    spdlog::error("Might an invalid wasm file");
     spdlog::error(ErrCode::Value::MalformedMagic);
     spdlog::error(
         "    The AOT compiled WASM shared library is not supported for loading "
@@ -183,12 +188,10 @@ Loader::parseWasmUnit(Span<const uint8_t> Code) {
 Expect<std::unique_ptr<AST::Module>>
 Loader::parseModule(const std::filesystem::path &FilePath) {
   if (auto R = parseWasmUnit(FilePath)) {
-    if (std::holds_alternative<AST::Module>(*R)) {
-      auto Res = std::make_unique<AST::Module>(std::get<AST::Module>(*R));
-      return Res;
-    } else {
-      return Unexpect(ErrCode::Value::MalformedVersion);
+    if (std::holds_alternative<std::unique_ptr<AST::Module>>(*R)) {
+      return std::move(std::get<std::unique_ptr<AST::Module>>(*R));
     }
+    return Unexpect(ErrCode::Value::MalformedVersion);
   } else {
     return Unexpect(R);
   }
@@ -198,63 +201,40 @@ Loader::parseModule(const std::filesystem::path &FilePath) {
 Expect<std::unique_ptr<AST::Module>>
 Loader::parseModule(Span<const uint8_t> Code) {
   if (auto R = parseWasmUnit(Code)) {
-    if (std::holds_alternative<AST::Module>(*R)) {
-      auto Res = std::make_unique<AST::Module>(std::get<AST::Module>(*R));
-      return Res;
-    } else {
-      return Unexpect(ErrCode::Value::MalformedVersion);
+    if (std::holds_alternative<std::unique_ptr<AST::Module>>(*R)) {
+      return std::move(std::get<std::unique_ptr<AST::Module>>(*R));
     }
+    return Unexpect(ErrCode::Value::MalformedVersion);
   } else {
     return Unexpect(R);
   }
 }
 
-// Helper function of checking the valid value types.
-Expect<ValType> Loader::checkValTypeProposals(ValType VType, uint64_t Off,
-                                              ASTNodeAttr Node) const noexcept {
-  if (VType == ValType::V128 && !Conf.hasProposal(Proposal::SIMD)) {
-    return logNeedProposal(ErrCode::Value::MalformedValType, Proposal::SIMD,
-                           Off, Node);
-  }
-  if ((VType == ValType::FuncRef &&
-       !Conf.hasProposal(Proposal::ReferenceTypes) &&
-       !Conf.hasProposal(Proposal::BulkMemoryOperations)) ||
-      (VType == ValType::ExternRef &&
-       !Conf.hasProposal(Proposal::ReferenceTypes))) {
-    return logNeedProposal(ErrCode::Value::MalformedElemType,
-                           Proposal::ReferenceTypes, Off, Node);
-  }
-  switch (VType) {
-  case ValType::I32:
-  case ValType::I64:
-  case ValType::F32:
-  case ValType::F64:
-  case ValType::V128:
-  case ValType::ExternRef:
-  case ValType::FuncRef:
-    return VType;
-  default:
-    return logLoadError(ErrCode::Value::MalformedValType, Off, Node);
-  }
+// Serialize module into byte code. See "include/loader/loader.h".
+Expect<std::vector<Byte>> Loader::serializeModule(const AST::Module &Mod) {
+  return Ser.serializeModule(Mod);
 }
 
-// Helper function of checking the valid reference types.
-Expect<RefType> Loader::checkRefTypeProposals(RefType RType, uint64_t Off,
-                                              ASTNodeAttr Node) const noexcept {
-  switch (RType) {
-  case RefType::ExternRef:
-    if (!Conf.hasProposal(Proposal::ReferenceTypes)) {
-      return logNeedProposal(ErrCode::Value::MalformedElemType,
-                             Proposal::ReferenceTypes, Off, Node);
+// Helper function to set the function type for tag.
+void Loader::setTagFunctionType(AST::TagSection &TagSec,
+                                AST::ImportSection &ImportSec,
+                                AST::TypeSection &TypeSec) {
+  auto &TypeVec = TypeSec.getContent();
+  for (auto &TgType : TagSec.getContent()) {
+    auto TypeIdx = TgType.getTypeIdx();
+    // Invalid type index would be checked during validation.
+    if (TypeIdx < TypeVec.size()) {
+      TgType.setDefType(&TypeVec[TypeIdx]);
     }
-    [[fallthrough]];
-  case RefType::FuncRef:
-    return RType;
-  default:
-    if (Conf.hasProposal(Proposal::ReferenceTypes)) {
-      return logLoadError(ErrCode::Value::MalformedRefType, Off, Node);
-    } else {
-      return logLoadError(ErrCode::Value::MalformedElemType, Off, Node);
+  }
+  for (auto &Desc : ImportSec.getContent()) {
+    if (Desc.getExternalType() == ExternalType::Tag) {
+      auto &TgType = Desc.getExternalTagType();
+      auto TypeIdx = TgType.getTypeIdx();
+      // Invalid type index would be checked during validation.
+      if (TypeIdx < TypeVec.size()) {
+        TgType.setDefType(&TypeVec[TypeIdx]);
+      }
     }
   }
 }
