@@ -14,6 +14,33 @@
 #include <unistd.h>
 #elif WASMEDGE_OS_WINDOWS
 #include "system/winapi.h"
+
+static bool GetFunctionModuleFileName(void *FuncPtr,
+                                      std::filesystem::path &Path) {
+  WasmEdge::winapi::HMODULE_ Module = nullptr;
+
+  if (!WasmEdge::winapi::GetModuleHandleExW(
+          WasmEdge::winapi::GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS_ |
+              WasmEdge::winapi::GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT_,
+          reinterpret_cast<WasmEdge::winapi::LPCWSTR_>(FuncPtr), &Module)) {
+    return false;
+  }
+
+  std::vector<wchar_t> Buffer;
+  WasmEdge::winapi::DWORD_ CopiedSize;
+  do {
+    Buffer.resize(Buffer.size() + WasmEdge::winapi::MAX_PATH_);
+    CopiedSize = WasmEdge::winapi::GetModuleFileNameW(
+        Module, Buffer.data(),
+        static_cast<WasmEdge::winapi::DWORD_>(Buffer.size()));
+    if (CopiedSize == 0) {
+      return false;
+    }
+  } while (CopiedSize >= Buffer.size());
+
+  Path.assign(std::wstring_view(Buffer.data(), CopiedSize));
+  return true;
+}
 #endif
 
 namespace WasmEdge {
@@ -36,32 +63,6 @@ template <> struct Parser<WasmEdge_String> {
 namespace Plugin {
 
 namespace {
-static unsigned int NiftyCounter = 0;
-static std::aligned_storage_t<sizeof(std::vector<Plugin>),
-                              alignof(std::vector<Plugin>)>
-    PluginRegistoryStorage;
-static std::aligned_storage_t<
-    sizeof(std::unordered_map<std::string_view, std::size_t>),
-    alignof(std::unordered_map<std::string_view, std::size_t>)>
-    PluginNameLookupStorage;
-
-void IncreaseNiftyCounter() noexcept {
-  if (NiftyCounter++ == 0) {
-    new (&PluginRegistoryStorage) std::vector<Plugin>();
-    new (&PluginNameLookupStorage)
-        std::unordered_map<std::string_view, std::size_t>();
-  }
-}
-
-void DecreaseNiftyCounter() noexcept {
-  if (--NiftyCounter == 0) {
-    reinterpret_cast<std::vector<Plugin> &>(PluginRegistoryStorage)
-        .~vector<Plugin>();
-    reinterpret_cast<std::unordered_map<std::string_view, std::size_t> &>(
-        PluginNameLookupStorage)
-        .~unordered_map<std::string_view, std::size_t>();
-  }
-}
 
 class CAPIPluginRegister {
 public:
@@ -69,8 +70,6 @@ public:
   CAPIPluginRegister &operator=(const CAPIPluginRegister &) = delete;
 
   CAPIPluginRegister(const WasmEdge_PluginDescriptor *Desc) noexcept {
-    IncreaseNiftyCounter();
-
     ModuleDescriptions.resize(Desc->ModuleCount);
     for (size_t I = 0; I < ModuleDescriptions.size(); ++I) {
       ModuleDescriptions[I].Name = Desc->ModuleDescriptions[I].Name;
@@ -167,9 +166,9 @@ public:
       }
     }
 
-    Plugin::registerPlugin(&Descriptor);
+    Result = Plugin::registerPlugin(&Descriptor);
   }
-  ~CAPIPluginRegister() noexcept { DecreaseNiftyCounter(); }
+  bool result() const noexcept { return Result; }
 
 private:
   static Runtime::Instance::ModuleInstance *
@@ -213,6 +212,8 @@ private:
   static std::unordered_map<const PluginModule::ModuleDescriptor *,
                             const WasmEdge_ModuleDescriptor *>
       DescriptionLookup;
+
+  bool Result = false;
 };
 std::unordered_map<const PluginModule::ModuleDescriptor *,
                    const WasmEdge_ModuleDescriptor *>
@@ -222,11 +223,9 @@ std::vector<std::unique_ptr<CAPIPluginRegister>> CAPIPluginRegisters;
 
 } // namespace
 
-std::vector<Plugin> &Plugin::PluginRegistory =
-    reinterpret_cast<std::vector<Plugin> &>(PluginRegistoryStorage);
-std::unordered_map<std::string_view, std::size_t> &Plugin::PluginNameLookup =
-    reinterpret_cast<std::unordered_map<std::string_view, std::size_t> &>(
-        PluginNameLookupStorage);
+std::vector<Plugin> WasmEdge::Plugin::Plugin::PluginRegistry;
+std::unordered_map<std::string_view, std::size_t>
+    WasmEdge::Plugin::Plugin::PluginNameLookup;
 
 std::vector<std::filesystem::path> Plugin::getDefaultPluginPaths() noexcept {
   using namespace std::literals::string_view_literals;
@@ -251,6 +250,12 @@ std::vector<std::filesystem::path> Plugin::getDefaultPluginPaths() noexcept {
   int Status =
       dladdr(reinterpret_cast<void *>(Plugin::getDefaultPluginPaths), &DLInfo);
   if (Status != 0) {
+    if (DLInfo.dli_fname == nullptr) {
+      spdlog::error(
+          "Address matched to a shared object but not to any symbol "sv
+          "within the object. dli_fname is null."sv);
+      return std::vector<std::filesystem::path>();
+    }
     auto LibPath = std::filesystem::u8path(DLInfo.dli_fname)
                        .parent_path()
                        .lexically_normal();
@@ -272,11 +277,22 @@ std::vector<std::filesystem::path> Plugin::getDefaultPluginPaths() noexcept {
       Result.push_back(LibPath / std::filesystem::u8path(".."sv) /
                        std::filesystem::u8path("plugin"sv));
     }
+  } else {
+    spdlog::error(ErrCode::Value::NonNullRequired);
+    spdlog::error("Address could not be matched to any shared object. "sv
+                  "Detailed error information is not available."sv);
+    return std::vector<std::filesystem::path>();
   }
 #elif WASMEDGE_OS_WINDOWS
-  // FIXME: Use the `dladdr`.
   // Global plugin directory.
-  Result.push_back(std::filesystem::u8path(kGlobalPluginDir));
+  if (std::filesystem::path Path; GetFunctionModuleFileName(
+          reinterpret_cast<void *>(Plugin::getDefaultPluginPaths), Path)) {
+    Result.push_back(Path.parent_path());
+  } else {
+    spdlog::error("Failed to get the path of the current module."sv);
+    return std::vector<std::filesystem::path>();
+  }
+
   // Local home plugin directory.
   std::filesystem::path Home;
   if (const auto HomeEnv = ::getenv("USERPROFILE")) {
@@ -332,14 +348,18 @@ WASMEDGE_EXPORT bool Plugin::load(const std::filesystem::path &Path) noexcept {
 }
 
 bool Plugin::loadFile(const std::filesystem::path &Path) noexcept {
-  const auto Index = PluginRegistory.size();
-
+  bool Result = false;
   auto Lib = std::make_shared<Loader::SharedLibrary>();
   if (auto Res = Lib->load(Path); unlikely(!Res)) {
     return false;
   }
 
-  if (PluginRegistory.size() != Index + 1) {
+  if (auto GetDescriptor =
+          Lib->get<Plugin::PluginDescriptor const *()>("GetDescriptor")) {
+    Result = Plugin::registerPlugin(GetDescriptor());
+  }
+
+  if (!Result) {
     // Check C interface
     if (auto GetDescriptor = Lib->get<decltype(WasmEdge_Plugin_GetDescriptor)>(
             "WasmEdge_Plugin_GetDescriptor");
@@ -349,19 +369,25 @@ bool Plugin::loadFile(const std::filesystem::path &Path) noexcept {
                unlikely(!Descriptor)) {
       return false;
     } else {
-      CAPIPluginRegisters.push_back(
-          std::make_unique<CAPIPluginRegister>(Descriptor));
+      Result =
+          CAPIPluginRegisters
+              .emplace_back(std::make_unique<CAPIPluginRegister>(Descriptor))
+              ->result();
     }
   }
 
-  auto &Plugin = PluginRegistory.back();
+  if (!Result) {
+    return false;
+  }
+
+  auto &Plugin = PluginRegistry.back();
   Plugin.Path = Path;
   Plugin.Lib = std::move(Lib);
   return true;
 }
 
 void Plugin::addPluginOptions(PO::ArgumentParser &Parser) noexcept {
-  for (const auto &Plugin : PluginRegistory) {
+  for (const auto &Plugin : PluginRegistry) {
     if (Plugin.Desc->AddOptions) {
       Plugin.Desc->AddOptions(Plugin.Desc, Parser);
     }
@@ -369,36 +395,39 @@ void Plugin::addPluginOptions(PO::ArgumentParser &Parser) noexcept {
 }
 
 WASMEDGE_EXPORT const Plugin *Plugin::find(std::string_view Name) noexcept {
-  if (NiftyCounter != 0) {
-    if (auto Iter = PluginNameLookup.find(Name);
-        Iter != PluginNameLookup.end()) {
-      return std::addressof(PluginRegistory[Iter->second]);
-    }
+  if (auto Iter = PluginNameLookup.find(Name); Iter != PluginNameLookup.end()) {
+    return std::addressof(PluginRegistry[Iter->second]);
   }
   return nullptr;
 }
 
-Span<const Plugin> Plugin::plugins() noexcept { return PluginRegistory; }
+Span<const Plugin> Plugin::plugins() noexcept { return PluginRegistry; }
 
-WASMEDGE_EXPORT void
+WASMEDGE_EXPORT bool
 Plugin::registerPlugin(const PluginDescriptor *Desc) noexcept {
-  assuming(NiftyCounter != 0);
   if (Desc->APIVersion != CurrentAPIVersion) {
-    return;
+    spdlog::debug(
+        "Plugin: API version {} of plugin {} is not match to current {}."sv,
+        Desc->APIVersion, Desc->Name, CurrentAPIVersion);
+    return false;
+  }
+  if (PluginNameLookup.find(Desc->Name) != PluginNameLookup.end()) {
+    spdlog::debug("Plugin: {} has already loaded."sv, Desc->Name);
+    return false;
   }
 
-  const auto Index = PluginRegistory.size();
-  PluginRegistory.push_back(Plugin(Desc));
+  const auto Index = PluginRegistry.size();
+  PluginRegistry.emplace_back(Desc);
   PluginNameLookup.emplace(Desc->Name, Index);
 
-  return;
+  return true;
 }
 
 Plugin::Plugin(const PluginDescriptor *D) noexcept : Desc(D) {
   for (const auto &ModuleDesc : Span<const PluginModule::ModuleDescriptor>(
            D->ModuleDescriptions, D->ModuleCount)) {
-    const auto Index = ModuleRegistory.size();
-    ModuleRegistory.push_back(PluginModule(&ModuleDesc));
+    const auto Index = ModuleRegistry.size();
+    ModuleRegistry.push_back(PluginModule(&ModuleDesc));
     ModuleNameLookup.emplace(ModuleDesc.Name, Index);
   }
 }
@@ -406,20 +435,9 @@ Plugin::Plugin(const PluginDescriptor *D) noexcept : Desc(D) {
 WASMEDGE_EXPORT const PluginModule *
 Plugin::findModule(std::string_view Name) const noexcept {
   if (auto Iter = ModuleNameLookup.find(Name); Iter != ModuleNameLookup.end()) {
-    return std::addressof(ModuleRegistory[Iter->second]);
+    return std::addressof(ModuleRegistry[Iter->second]);
   }
   return nullptr;
 }
-
-WASMEDGE_EXPORT
-PluginRegister::PluginRegister(const Plugin::PluginDescriptor *Desc) noexcept {
-  IncreaseNiftyCounter();
-  Plugin::registerPlugin(Desc);
-}
-
-WASMEDGE_EXPORT PluginRegister::~PluginRegister() noexcept {
-  DecreaseNiftyCounter();
-}
-
 } // namespace Plugin
 } // namespace WasmEdge
