@@ -4,10 +4,9 @@
 #include "validator/validator.h"
 
 #include "common/errinfo.h"
-#include "common/log.h"
 
-#include <array>
 #include <cstdint>
+#include <numeric>
 #include <string>
 #include <unordered_set>
 #include <vector>
@@ -15,9 +14,24 @@
 namespace WasmEdge {
 namespace Validator {
 
-Expect<void> Validator::validate(const AST::Component &) {
-  spdlog::error("component validation is not done yet.");
-  return Unexpect(ErrCode::Value::RuntimeError);
+Expect<void> Validator::validate(const AST::Component::Component &Comp) {
+  using namespace AST::Component;
+
+  spdlog::warn("component validation is not done yet.");
+
+  for (auto &Sec : Comp.getSections()) {
+    if (std::holds_alternative<AST::CoreModuleSection>(Sec)) {
+      auto &Mod = std::get<AST::CoreModuleSection>(Sec).getContent();
+      validate(Mod);
+    } else if (std::holds_alternative<ComponentSection>(Sec)) {
+      auto &C = std::get<ComponentSection>(Sec).getContent();
+      validate(C);
+    } else {
+      // TODO: validate others section
+    }
+  }
+
+  return {};
 }
 
 // Validate Module. See "include/validator/validator.h".
@@ -25,9 +39,11 @@ Expect<void> Validator::validate(const AST::Module &Mod) {
   // https://webassembly.github.io/spec/core/valid/modules.html
   Checker.reset(true);
 
-  // Register type definitions into FormChecker.
-  for (auto &Type : Mod.getTypeSection().getContent()) {
-    Checker.addType(Type);
+  // Validate and register type section.
+  if (auto Res = validate(Mod.getTypeSection()); !Res) {
+    spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Sec_Type));
+    spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Module));
+    return Unexpect(Res);
   }
 
   // Validate and register import section into FormChecker.
@@ -65,6 +81,13 @@ Expect<void> Validator::validate(const AST::Module &Mod) {
     return Unexpect(Res);
   }
 
+  // Validate tag section and register tags into FormChecker.
+  if (auto Res = validate(Mod.getTagSection()); !Res) {
+    spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Sec_Tag));
+    spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Module));
+    return Unexpect(Res);
+  }
+
   // Validate export section.
   if (auto Res = validate(Mod.getExportSection()); !Res) {
     spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Sec_Export));
@@ -122,6 +145,64 @@ Expect<void> Validator::validate(const AST::Module &Mod) {
   return {};
 }
 
+// Validate Sub type. See "include/validator/validator.h".
+Expect<void> Validator::validate(const AST::SubType &Type) {
+  const auto &TypeVec = Checker.getTypes();
+  const auto &CompType = Type.getCompositeType();
+
+  // Check the validation of the composite type.
+  if (CompType.isFunc()) {
+    const auto &FType = CompType.getFuncType();
+    for (auto &PType : FType.getParamTypes()) {
+      if (auto Res = Checker.validate(PType); !Res) {
+        spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Function));
+        return Unexpect(Res);
+      }
+    }
+    for (auto &RType : FType.getReturnTypes()) {
+      if (auto Res = Checker.validate(RType); !Res) {
+        spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Function));
+        return Unexpect(Res);
+      }
+    }
+  } else {
+    const auto &FTypes = CompType.getFieldTypes();
+    for (auto &FieldType : FTypes) {
+      if (auto Res = Checker.validate(FieldType.getStorageType()); !Res) {
+        return Unexpect(Res);
+      }
+    }
+  }
+
+  // In current version, the length of type index vector will be <= 1.
+  if (Type.getSuperTypeIndices().size() > 1) {
+    spdlog::error(ErrCode::Value::InvalidSubType);
+    spdlog::error("    Accepts 1 super type currently.");
+    return Unexpect(ErrCode::Value::InvalidSubType);
+  }
+  for (auto Index : Type.getSuperTypeIndices()) {
+    if (Index >= TypeVec.size()) {
+      spdlog::error(ErrCode::Value::InvalidSubType);
+      spdlog::error(
+          ErrInfo::InfoForbidIndex(ErrInfo::IndexCategory::DefinedType, Index,
+                                   static_cast<uint32_t>(TypeVec.size())));
+      return Unexpect(ErrCode::Value::InvalidSubType);
+    }
+    if (TypeVec[Index]->isFinal()) {
+      spdlog::error(ErrCode::Value::InvalidSubType);
+      spdlog::error("    Super type should not be final.");
+      return Unexpect(ErrCode::Value::InvalidSubType);
+    }
+    auto &SuperType = TypeVec[Index]->getCompositeType();
+    if (!AST::TypeMatcher::matchType(Checker.getTypes(), SuperType, CompType)) {
+      spdlog::error(ErrCode::Value::InvalidSubType);
+      spdlog::error("    Super type not matched.");
+      return Unexpect(ErrCode::Value::InvalidSubType);
+    }
+  }
+  return {};
+}
+
 // Validate Limit type. See "include/validator/validator.h".
 Expect<void> Validator::validate(const AST::Limit &Lim) {
   if (Lim.hasMax() && Lim.getMin() > Lim.getMax()) {
@@ -134,8 +215,13 @@ Expect<void> Validator::validate(const AST::Limit &Lim) {
 
 // Validate Table type. See "include/validator/validator.h".
 Expect<void> Validator::validate(const AST::TableType &Tab) {
+  // Validate value type.
+  if (auto Res = Checker.validate(Tab.getRefType()); !Res) {
+    return Unexpect(Res);
+  }
   // Validate table limits.
   if (auto Res = validate(Tab.getLimit()); !Res) {
+    spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Limit));
     return Unexpect(Res);
   }
   return {};
@@ -146,6 +232,7 @@ Expect<void> Validator::validate(const AST::MemoryType &Mem) {
   // Validate memory limits.
   const auto &Lim = Mem.getLimit();
   if (auto Res = validate(Lim); !Res) {
+    spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Limit));
     return Unexpect(Res);
   }
   if (Lim.getMin() > LIMIT_MEMORYTYPE ||
@@ -157,6 +244,46 @@ Expect<void> Validator::validate(const AST::MemoryType &Mem) {
   return {};
 }
 
+// Validate Global type. See "include/validator/validator.h".
+Expect<void> Validator::validate(const AST::GlobalType &Glob) {
+  // Validate value type.
+  if (auto Res = Checker.validate(Glob.getValType()); !Res) {
+    return Unexpect(Res);
+  }
+  return {};
+}
+
+// Validate Table segment. See "include/validator/validator.h".
+Expect<void> Validator::validate(const AST::TableSegment &TabSeg) {
+  if (TabSeg.getExpr().getInstrs().size() > 0) {
+    // Check ref initialization is a const expression.
+    if (auto Res =
+            validateConstExpr(TabSeg.getExpr().getInstrs(),
+                              {ValType(TabSeg.getTableType().getRefType())});
+        !Res) {
+      spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Expression));
+      return Unexpect(Res);
+    }
+  } else {
+    // No init expression. Check the reference type is nullable.
+    if (!TabSeg.getTableType().getRefType().isNullableRefType()) {
+      spdlog::error(ErrCode::Value::TypeCheckFailed);
+      spdlog::error(ErrInfo::InfoMismatch(
+          ValType(TypeCode::RefNull,
+                  TabSeg.getTableType().getRefType().getHeapTypeCode(),
+                  TabSeg.getTableType().getRefType().getTypeIndex()),
+          TabSeg.getTableType().getRefType()));
+      return Unexpect(ErrCode::Value::TypeCheckFailed);
+    }
+  }
+  // Validate table type.
+  if (auto Res = validate(TabSeg.getTableType()); !Res) {
+    spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Table));
+    return Unexpect(Res);
+  }
+  return {};
+}
+
 // Validate Global segment. See "include/validator/validator.h".
 Expect<void> Validator::validate(const AST::GlobalSegment &GlobSeg) {
   // Check global initialization is a const expression.
@@ -166,6 +293,11 @@ Expect<void> Validator::validate(const AST::GlobalSegment &GlobSeg) {
     spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Expression));
     return Unexpect(Res);
   }
+  // Validate global type.
+  if (auto Res = validate(GlobSeg.getGlobalType()); !Res) {
+    spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Global));
+    return Unexpect(Res);
+  }
   return {};
 }
 
@@ -174,14 +306,19 @@ Expect<void> Validator::validate(const AST::ElementSegment &ElemSeg) {
   // Check initialization expressions are const expressions.
   for (auto &Expr : ElemSeg.getInitExprs()) {
     if (auto Res = validateConstExpr(Expr.getInstrs(),
-                                     {ToValType(ElemSeg.getRefType())});
+                                     {ValType(ElemSeg.getRefType())});
         !Res) {
       spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Expression));
       return Unexpect(Res);
     }
   }
 
-  // Passive and declarative cases are always valid with reference type.
+  // The reference type should be valid.
+  if (auto Res = Checker.validate(ElemSeg.getRefType()); !Res) {
+    return Unexpect(Res);
+  }
+
+  // Passive and declarative cases are valid with the valid reference type.
   if (ElemSeg.getMode() == AST::ElementSegment::ElemMode::Active) {
     // Check table index and reference type in context.
     const auto &TableVec = Checker.getTables();
@@ -192,17 +329,23 @@ Expect<void> Validator::validate(const AST::ElementSegment &ElemSeg) {
           static_cast<uint32_t>(TableVec.size())));
       return Unexpect(ErrCode::Value::InvalidTableIdx);
     }
-    if (TableVec[ElemSeg.getIdx()] != ElemSeg.getRefType()) {
+    // TODO: Use AST::TypeMatcher::matchType() to match types instead.
+    // For the element segments, the RefType may not record the strict type
+    // index, and should check the init exprs for the real type index to do type
+    // matching. But for the table type, the type index is recorded into the
+    // heap type. So it will fail here to do strict type matching. Therefore,
+    // only check the FuncRef and ExternRef here.
+    if (TableVec[ElemSeg.getIdx()].isFuncRefType() !=
+        ElemSeg.getRefType().isFuncRefType()) {
       // Reference type not matched.
       spdlog::error(ErrCode::Value::TypeCheckFailed);
-      spdlog::error(ErrInfo::InfoMismatch(
-          static_cast<ValType>(TableVec[ElemSeg.getIdx()]),
-          static_cast<ValType>(ElemSeg.getRefType())));
+      spdlog::error(ErrInfo::InfoMismatch(TableVec[ElemSeg.getIdx()],
+                                          ElemSeg.getRefType()));
       return Unexpect(ErrCode::Value::TypeCheckFailed);
     }
     // Check table initialization is a const expression.
-    if (auto Res =
-            validateConstExpr(ElemSeg.getExpr().getInstrs(), {ValType::I32});
+    if (auto Res = validateConstExpr(ElemSeg.getExpr().getInstrs(),
+                                     {ValType(TypeCode::I32)});
         !Res) {
       spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Expression));
       return Unexpect(Res);
@@ -214,21 +357,30 @@ Expect<void> Validator::validate(const AST::ElementSegment &ElemSeg) {
 // Validate Code segment. See "include/validator/validator.h".
 Expect<void> Validator::validate(const AST::CodeSegment &CodeSeg,
                                  const uint32_t TypeIdx) {
+  // Due to the validation of the function section, the type of index bust be a
+  // function type.
+  const auto &FuncType =
+      Checker.getTypes()[TypeIdx]->getCompositeType().getFuncType();
   // Reset stack in FormChecker.
   Checker.reset();
   // Add parameters into this frame.
-  for (auto Val : Checker.getTypes()[TypeIdx].first) {
-    Checker.addLocal(Val);
+  for (auto &Type : FuncType.getParamTypes()) {
+    // Local passed as function parameters should be initialized.
+    Checker.addLocal(Type, true);
   }
   // Add locals into this frame.
   for (auto Val : CodeSeg.getLocals()) {
     for (uint32_t Cnt = 0; Cnt < Val.first; ++Cnt) {
-      Checker.addLocal(Val.second);
+      // The local value type should be valid.
+      if (auto Res = Checker.validate(Val.second); !Res) {
+        return Unexpect(Res);
+      }
+      Checker.addLocal(Val.second, false);
     }
   }
   // Validate function body expression.
   if (auto Res = Checker.validate(CodeSeg.getExpr().getInstrs(),
-                                  Checker.getTypes()[TypeIdx].second);
+                                  FuncType.getReturnTypes());
       !Res) {
     spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Expression));
     return Unexpect(Res);
@@ -248,8 +400,8 @@ Expect<void> Validator::validate(const AST::DataSegment &DataSeg) {
       return Unexpect(ErrCode::Value::InvalidMemoryIdx);
     }
     // Check memory initialization is a const expression.
-    if (auto Res =
-            validateConstExpr(DataSeg.getExpr().getInstrs(), {ValType::I32});
+    if (auto Res = validateConstExpr(DataSeg.getExpr().getInstrs(),
+                                     {ValType(TypeCode::I32)});
         !Res) {
       spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Expression));
       return Unexpect(Res);
@@ -266,7 +418,7 @@ Expect<void> Validator::validate(const AST::ImportDesc &ImpDesc) {
   // loader phase.
   case ExternalType::Function: {
     const auto TId = ImpDesc.getExternalFuncTypeIdx();
-    // Function type index must exist in context.
+    // Function type index must exist in context and be valid.
     if (TId >= Checker.getTypes().size()) {
       spdlog::error(ErrCode::Value::InvalidFuncTypeIdx);
       spdlog::error(ErrInfo::InfoForbidIndex(
@@ -274,6 +426,11 @@ Expect<void> Validator::validate(const AST::ImportDesc &ImpDesc) {
           static_cast<uint32_t>(Checker.getTypes().size())));
       return Unexpect(ErrCode::Value::InvalidFuncTypeIdx);
     }
+    if (!Checker.getTypes()[TId]->getCompositeType().isFunc()) {
+      spdlog::error(ErrCode::Value::InvalidFuncTypeIdx);
+      spdlog::error("    Defined type index {} is not a function type.", TId);
+      return Unexpect(ErrCode::Value::InvalidFuncTypeIdx);
+    }
     Checker.addRef(static_cast<uint32_t>(Checker.getFunctions().size()));
     Checker.addFunc(TId, true);
     return {};
@@ -298,10 +455,30 @@ Expect<void> Validator::validate(const AST::ImportDesc &ImpDesc) {
     Checker.addMemory(MemType);
     return {};
   }
-  case ExternalType::Global:
-    // Global type always is valid.
-    Checker.addGlobal(ImpDesc.getExternalGlobalType(), true);
+  case ExternalType::Tag: {
+    const auto &T = ImpDesc.getExternalTagType();
+    // Tag type index must exist in context.
+    auto TagTypeIdx = T.getTypeIdx();
+    if (TagTypeIdx >= Checker.getTypes().size()) {
+      spdlog::error(ErrCode::Value::InvalidTagIdx);
+      spdlog::error(ErrInfo::InfoForbidIndex(
+          ErrInfo::IndexCategory::TagType, TagTypeIdx,
+          static_cast<uint32_t>(Checker.getTypes().size())));
+      return Unexpect(ErrCode::Value::InvalidTagIdx);
+    }
+    Checker.addTag(TagTypeIdx);
     return {};
+  }
+  case ExternalType::Global: {
+    const auto &GlobType = ImpDesc.getExternalGlobalType();
+    // Global type must be valid.
+    if (auto Res = validate(GlobType); !Res) {
+      spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Global));
+      return Unexpect(Res);
+    }
+    Checker.addGlobal(GlobType, true);
+    return {};
+  }
   default:
     break;
   }
@@ -339,6 +516,15 @@ Expect<void> Validator::validate(const AST::ExportDesc &ExpDesc) {
       return Unexpect(ErrCode::Value::InvalidMemoryIdx);
     }
     return {};
+  case ExternalType::Tag:
+    if (Id >= Checker.getTags().size()) {
+      spdlog::error(ErrCode::Value::InvalidTagIdx);
+      spdlog::error(ErrInfo::InfoForbidIndex(
+          ErrInfo::IndexCategory::Tag, Id,
+          static_cast<uint32_t>(Checker.getTags().size())));
+      return Unexpect(ErrCode::Value::InvalidTagIdx);
+    }
+    return {};
   case ExternalType::Global:
     if (Id >= Checker.getGlobals().size()) {
       spdlog::error(ErrCode::Value::InvalidGlobalIdx);
@@ -354,6 +540,46 @@ Expect<void> Validator::validate(const AST::ExportDesc &ExpDesc) {
   return {};
 }
 
+Expect<void> Validator::validate(const AST::TypeSection &TypeSec) {
+  const auto STypeList = TypeSec.getContent();
+  uint32_t Idx = 0;
+  while (Idx < STypeList.size()) {
+    const auto &SType = STypeList[Idx];
+    if (SType.getRecursiveInfo().has_value()) {
+      // Recursive type case. Add types first for referring recursively.
+      uint32_t RecSize = SType.getRecursiveInfo()->RecTypeSize;
+      for (uint32_t I = Idx; I < Idx + RecSize; I++) {
+        Checker.addType(STypeList[I]);
+      }
+      for (uint32_t I = Idx; I < Idx + RecSize; I++) {
+        if (auto Res = validate(STypeList[I]); !Res) {
+          spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Rec));
+          return Unexpect(Res);
+        }
+      }
+      Idx += RecSize;
+    } else {
+      // SubType case.
+      if (Conf.hasProposal(Proposal::GC)) {
+        // For the GC proposal, the subtype is seemed as a self-recursive type.
+        // Add types first for referring recursively.
+        Checker.addType(SType);
+        if (auto Res = validate(*Checker.getTypes().back()); !Res) {
+          return Unexpect(Res);
+        }
+      } else {
+        // Validating first.
+        if (auto Res = validate(SType); !Res) {
+          return Unexpect(Res);
+        }
+        Checker.addType(SType);
+      }
+      Idx++;
+    }
+  }
+  return {};
+}
+
 // Validate Import section. See "include/validator/validator.h".
 Expect<void> Validator::validate(const AST::ImportSection &ImportSec) {
   for (auto &ImportDesc : ImportSec.getContent()) {
@@ -379,6 +605,11 @@ Expect<void> Validator::validate(const AST::FunctionSection &FuncSec) {
                                    static_cast<uint32_t>(TypeVec.size())));
       return Unexpect(ErrCode::Value::InvalidFuncTypeIdx);
     }
+    if (!TypeVec[TId]->getCompositeType().isFunc()) {
+      spdlog::error(ErrCode::Value::InvalidFuncTypeIdx);
+      spdlog::error("    Defined type index {} is not a function type.", TId);
+      return Unexpect(ErrCode::Value::InvalidFuncTypeIdx);
+    }
     Checker.addFunc(TId);
   }
   return {};
@@ -388,9 +619,9 @@ Expect<void> Validator::validate(const AST::FunctionSection &FuncSec) {
 Expect<void> Validator::validate(const AST::TableSection &TabSec) {
   for (auto &Tab : TabSec.getContent()) {
     if (auto Res = validate(Tab)) {
-      Checker.addTable(Tab);
+      Checker.addTable(Tab.getTableType());
     } else {
-      spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Table));
+      spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Seg_Table));
       return Unexpect(Res);
     }
   }
@@ -416,7 +647,7 @@ Expect<void> Validator::validate(const AST::GlobalSection &GlobSec) {
     if (auto Res = validate(GlobSeg)) {
       Checker.addGlobal(GlobSeg.getGlobalType());
     } else {
-      spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Type_Global));
+      spdlog::error(ErrInfo::InfoAST(ASTNodeAttr::Seg_Global));
       return Unexpect(Res);
     }
   }
@@ -485,18 +716,18 @@ Expect<void> Validator::validate(const AST::StartSection &StartSec) {
       return Unexpect(ErrCode::Value::InvalidFuncIdx);
     }
     auto TId = Checker.getFunctions()[FId];
-    auto &Type = Checker.getTypes()[TId];
-    if (Type.first.size() != 0 || Type.second.size() != 0) {
-      // Start function signature should be {}->{}
-      std::vector<ValType> Params, Returns;
-      for (auto &V : Type.first) {
-        Params.push_back(Checker.VTypeToAST(V));
-      }
-      for (auto &V : Type.second) {
-        Returns.push_back(Checker.VTypeToAST(V));
-      }
+    assuming(TId < Checker.getTypes().size());
+    if (!Checker.getTypes()[TId]->getCompositeType().isFunc()) {
       spdlog::error(ErrCode::Value::InvalidStartFunc);
-      spdlog::error(ErrInfo::InfoMismatch({}, {}, Params, Returns));
+      spdlog::error("    Defined type index {} is not a function type.", TId);
+      return Unexpect(ErrCode::Value::InvalidStartFunc);
+    }
+    auto &Type = Checker.getTypes()[TId]->getCompositeType().getFuncType();
+    if (Type.getParamTypes().size() != 0 || Type.getReturnTypes().size() != 0) {
+      // Start function signature should be {}->{}
+      spdlog::error(ErrCode::Value::InvalidStartFunc);
+      spdlog::error(ErrInfo::InfoMismatch({}, {}, Type.getParamTypes(),
+                                          Type.getReturnTypes()));
       return Unexpect(ErrCode::Value::InvalidStartFunc);
     }
   }
@@ -522,6 +753,37 @@ Expect<void> Validator::validate(const AST::ExportSection &ExportSec) {
   return {};
 }
 
+// Validate Tag section. See "include/validator/validator.h".
+Expect<void> Validator::validate(const AST::TagSection &TagSec) {
+  const auto &TagVec = TagSec.getContent();
+  const auto &TypeVec = Checker.getTypes();
+
+  // Check if type id of tag is valid in context.
+  for (auto &TagType : TagVec) {
+    auto TagTypeIdx = TagType.getTypeIdx();
+    if (TagTypeIdx >= TypeVec.size()) {
+      spdlog::error(ErrCode::Value::InvalidTagIdx);
+      spdlog::error(
+          ErrInfo::InfoForbidIndex(ErrInfo::IndexCategory::TagType, TagTypeIdx,
+                                   static_cast<uint32_t>(TypeVec.size())));
+      return Unexpect(ErrCode::Value::InvalidTagIdx);
+    }
+    auto &CompType = TypeVec[TagTypeIdx]->getCompositeType();
+    if (!CompType.isFunc()) {
+      spdlog::error(ErrCode::Value::InvalidTagIdx);
+      spdlog::error("    Defined type index {} is not a function type.",
+                    TagTypeIdx);
+      return Unexpect(ErrCode::Value::InvalidTagIdx);
+    }
+    if (!CompType.getFuncType().getReturnTypes().empty()) {
+      spdlog::error(ErrCode::Value::InvalidTagResultType);
+      return Unexpect(ErrCode::Value::InvalidTagResultType);
+    }
+    Checker.addTag(TagTypeIdx);
+  }
+  return {};
+}
+
 // Validate constant expression. See "include/validator/validator.h".
 Expect<void> Validator::validateConstExpr(AST::InstrView Instrs,
                                           Span<const ValType> Returns) {
@@ -531,11 +793,14 @@ Expect<void> Validator::validateConstExpr(AST::InstrView Instrs,
     case OpCode::Global__get: {
       // For initialization case, global indices must be imported globals.
       auto GlobIdx = Instr.getTargetIndex();
-      if (GlobIdx >= Checker.getNumImportGlobals()) {
+      uint32_t ValidGlobalSize = Checker.getNumImportGlobals();
+      if (Conf.hasProposal(Proposal::GC)) {
+        ValidGlobalSize = static_cast<uint32_t>(Checker.getGlobals().size());
+      }
+      if (GlobIdx >= ValidGlobalSize) {
         spdlog::error(ErrCode::Value::InvalidGlobalIdx);
-        spdlog::error(ErrInfo::InfoForbidIndex(
-            ErrInfo::IndexCategory::Global, GlobIdx,
-            static_cast<uint32_t>(Checker.getNumImportGlobals())));
+        spdlog::error(ErrInfo::InfoForbidIndex(ErrInfo::IndexCategory::Global,
+                                               GlobIdx, ValidGlobalSize));
         spdlog::error(
             ErrInfo::InfoInstruction(Instr.getOpCode(), Instr.getOffset()));
         return Unexpect(ErrCode::Value::InvalidGlobalIdx);
@@ -571,6 +836,14 @@ Expect<void> Validator::validateConstExpr(AST::InstrView Instrs,
     case OpCode::Ref__null:
     case OpCode::V128__const:
     case OpCode::End:
+    case OpCode::Struct__new:
+    case OpCode::Struct__new_default:
+    case OpCode::Array__new:
+    case OpCode::Array__new_default:
+    case OpCode::Array__new_fixed:
+    case OpCode::Any__convert_extern:
+    case OpCode::Extern__convert_any:
+    case OpCode::Ref__i31:
       break;
 
     // For the Extended-const proposal, these instructions are accepted.
