@@ -32,9 +32,8 @@ Plugin::Plugin::PluginDescriptor Descriptor{
     .AddOptions = nullptr,
 };
 
+EXPORT_GET_DESCRIPTOR(Descriptor)
+
 } // namespace
-
-Plugin::PluginRegister WasmEdgeTensorflowLite::TFLiteEnv::Register(&Descriptor);
-
 } // namespace Host
 } // namespace WasmEdge
