@@ -31,9 +31,9 @@
 namespace {
 WasmEdge::Runtime::Instance::ModuleInstance *createModule() {
   using namespace std::literals::string_view_literals;
-  WasmEdge::Plugin::Plugin::load(std::filesystem::u8path(
-      "../../../plugins/wasm_bpf/"
-      "libwasmedgePluginWasmBpf" WASMEDGE_LIB_EXTENSION));
+  WasmEdge::Plugin::Plugin::load(
+      std::filesystem::u8path("../../../plugins/wasm_bpf/" WASMEDGE_LIB_PREFIX
+                              "wasmedgePluginWasmBpf" WASMEDGE_LIB_EXTENSION));
   if (const auto *Plugin = WasmEdge::Plugin::Plugin::find("wasm_bpf"sv)) {
     if (const auto *Module = Plugin->findModule("wasm_bpf"sv)) {
       return Module->create().release();
@@ -279,17 +279,17 @@ TEST(WasmBpfTest, RunBpfProgramWithPolling) {
 
   // In the following several steps we will prepare for polling
   // Create an instance of the polling callback function
-  auto callbackFuncInst =
-      std::make_unique<WasmEdge::Runtime::Instance::FunctionInstance>(
-          &moduleInst, std::make_unique<PollCallbackFunction>());
+  moduleInst.addHostFunc("__polling_callback_hostfunc"sv,
+                         std::make_unique<PollCallbackFunction>());
+  auto *callbackFuncInst =
+      moduleInst.findFuncExports("__polling_callback_hostfunc");
   // Create a function table, and fill the callback function into it
   auto funcTableInst =
       std::make_unique<WasmEdge::Runtime::Instance::TableInstance>(
-          WasmEdge::AST::TableType(WasmEdge::RefType::FuncRef, 1));
+          WasmEdge::AST::TableType(WasmEdge::TypeCode::FuncRef, 1));
   EXPECT_TRUE(funcTableInst->setRefs(
-      std::initializer_list<const WasmEdge::RefVariant>{
-          WasmEdge::FuncRef(callbackFuncInst.get())},
-      0, 0, 1));
+      std::initializer_list<const WasmEdge::RefVariant>{callbackFuncInst}, 0, 0,
+      1));
   // Add the table to the main module
   moduleInst.addHostTable("__indirect_function_table"sv,
                           std::move(funcTableInst));
